package com.ruoyi.zdsz.domain.bo;

import com.ruoyi.common.core.validate.AddGroup;
import com.ruoyi.common.core.validate.EditGroup;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;

import javax.validation.Valid;
import javax.validation.constraints.*;

import java.util.Date;

import java.util.Date;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.ruoyi.common.core.domain.BaseEntity;
import lombok.NoArgsConstructor;
import org.springframework.validation.annotation.Validated;

/**
 * 工程详情业务对象 z_engineering_info
 *
 * @author ruoyi
 * @date 2023-12-27
 */

@Data
@EqualsAndHashCode(callSuper = true)
@NoArgsConstructor
public class ZEngineeringInfoBo extends BaseEntity {

    /**
     *
     */
    @NotBlank(message = "不能为空", groups = { EditGroup.class })
    private String id;

    /**
     * 民用工程id
     */
    @NotBlank(message = "民用工程id不能为空", groups = { AddGroup.class, EditGroup.class })
    private String engInfoId;

    /**
     * 施工地址
     */
    private String constructAddre;

    /**
     * 是否按图纸施工
     */
    private String constructAccordingDrawings;

    /**
     * 分段打压是否合格
     */
    private String segmentedCompressionQualified;

    /**
     * 回填时间
     */
    private Date backfillTime;

    /**
     * 施工人电话
     */
    private String constructPhone;

    /**
     * 施工人
     */
    private String constructUser;

    /**
     * 施工时间
     */
    private Date constructTime;

    /**
     * 负责人名称
     */
    private String headName;

    /**
     * 负责人电话
     */
    private String headPhone;


    /**
     * 备注
     */
    private String remark;

    public List<String> getzEngiineeringPhotoBoList() {
        return zEngiineeringPhotoBoList;
    }

    public void setzEngiineeringPhotoBoList(List<String> zEngiineeringPhotoBoList) {
        this.zEngiineeringPhotoBoList = zEngiineeringPhotoBoList;
    }

    /**
     * 照片
     */
    @Valid
    @NotNull(message = "照片不能为空", groups = { AddGroup.class, EditGroup.class })
    private List<String> zEngiineeringPhotoBoList;


    public List<ZEngineeringMaterialBo> getzEngineeringMaterialBo() {
        return zEngineeringMaterialBo;
    }

    public void setzEngineeringMaterialBo(List<ZEngineeringMaterialBo> zEngineeringMaterialBo) {
        this.zEngineeringMaterialBo = zEngineeringMaterialBo;
    }

    /**
     * 工程用料
     */

    private  List<ZEngineeringMaterialBo> zEngineeringMaterialBo;

    public ZEngineeringReviewBo getzEngineeringReviewBo() {
        return zEngineeringReviewBo;
    }

    public void setzEngineeringReviewBo(ZEngineeringReviewBo zEngineeringReviewBo) {
        this.zEngineeringReviewBo = zEngineeringReviewBo;
    }

    /**
     * 审核信息
     */
    private  ZEngineeringReviewBo zEngineeringReviewBo;

}
